import { EventEmitter, AfterViewInit } from '@angular/core';
import { AbstractControl } from '@angular/forms';
import { ExtendedFormGroup } from '../../forms';
import { BaseContainer } from '../base-container/base-container.component';
/**
 * Represents the base component for all form component.
 * It provides getter and setter for common attributes.
 */
export declare abstract class BaseControl extends BaseContainer implements AfterViewInit {
    form: ExtendedFormGroup;
    label: string;
    value: any;
    postBack: boolean;
    debounceTime: number;
    hidden: boolean;
    focus: boolean;
    emitEventAfterViewInit: boolean;
    required: boolean;
    readonly: boolean;
    isCultureSpecific: boolean;
    onChange: EventEmitter<{}>;
    onAfterViewInit: EventEmitter<{}>;
    private _onPostBackRequest;
    abstract control: any;
    ngAfterViewInit(): void;
    protected readonly formControl: AbstractControl;
    readonly isValid: boolean;
    readonly backupValue: any;
    /**
     * Sync data from view to form model, as well as mark a field as dirty.
     * @param value the field value got from view's input
     */
    valueChange(value: any): void;
    /**
     * The default implementation of value comparision.
     * This function is called to determine if the value is changed, to mark the field
     * as dirty or pristine.
     * Implementation can override this, to do custom comparision. For example, to treat null
     * and empty string equally.
     * @param newValue new value.
     * @param oldValue old value.
     */
    isEqual(newValue: any, oldValue: any): boolean;
}
