import { AfterViewInit, ChangeDetectorRef, ElementRef, EventEmitter, OnInit } from '@angular/core';
import { Observable } from 'rxjs';
import { BaseControl } from '../base-control/base-control.component';
import { Field } from '../../interfaces/field';
/**
 * Represents the base class for field editor.
 * BaseFieldEditor contains helper functions to get the correct language for editing
 */
export declare abstract class BaseFieldEditor extends BaseControl implements OnInit, AfterViewInit, Field {
    protected _changeDetection: ChangeDetectorRef;
    private _editLanguage;
    private _previewLanguage;
    private _viewLanguage;
    valueAsDictionary: boolean;
    tooltip: string;
    constructor(_changeDetection: ChangeDetectorRef);
    editable: boolean;
    onBeforeUndo: EventEmitter<{}>;
    onUndo: EventEmitter<{}>;
    onValueSet: EventEmitter<{}>;
    control: ElementRef;
    previewLanguage$: Observable<string>;
    editAll$: Observable<boolean>;
    ngOnInit(): void;
    ngAfterViewInit(): void;
    /**
     * Sync data from view to form model, as well as mark a field as dirty.
     * @param value the field value got from view's input
     * @param editLanguage the current edit language
     */
    valueChange(value: any, editLanguage?: string): void;
    getValue(language: string): any;
    value: any;
    editLanguage: string;
    viewLanguage: string;
    previewLanguage: string;
    undo(): void;
    readonly dirty: boolean;
    readonly field: Field;
    onEditLanguageChanged(): void;
}
