var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { ElementRef, EventEmitter, Input, Output, ViewChild } from '@angular/core';
import cloneDeep from 'lodash-es/cloneDeep';
import { select } from '@angular-redux/store';
import { Observable } from 'rxjs';
import { BaseControl } from '../base-control/base-control.component';
/**
 * Represents the base class for field editor.
 * BaseFieldEditor contains helper functions to get the correct language for editing
 */
var BaseFieldEditor = /** @class */ (function (_super) {
    __extends(BaseFieldEditor, _super);
    function BaseFieldEditor(_changeDetection) {
        var _this = _super.call(this) || this;
        _this._changeDetection = _changeDetection;
        _this._editLanguage = null;
        _this._previewLanguage = '*';
        _this._viewLanguage = '*';
        _this.valueAsDictionary = false;
        _this.onBeforeUndo = new EventEmitter();
        _this.onUndo = new EventEmitter();
        _this.onValueSet = new EventEmitter();
        return _this;
    }
    Object.defineProperty(BaseFieldEditor.prototype, "editable", {
        get: function () {
            return !this.readonly && this.form.controls[this.name] && this.form.controls[this.name]['editable'] || false;
        },
        set: function (value) {
            if (this.form.controls[this.name]) {
                this.form.controls[this.name]['editable'] = value;
                this._changeDetection && this._changeDetection.markForCheck();
            }
        },
        enumerable: true,
        configurable: true
    });
    BaseFieldEditor.prototype.ngOnInit = function () {
        var _this = this;
        this.on(this.previewLanguage$, function (previewLanguage) { return _this.previewLanguage = previewLanguage; });
        this.on(this.form.onPristine, function () { return _this._changeDetection && _this._changeDetection.markForCheck(); });
        this.on(this.editAll$, function () { return _this._changeDetection && _this._changeDetection.markForCheck(); });
    };
    BaseFieldEditor.prototype.ngAfterViewInit = function () {
        var _this = this;
        setTimeout(function () { return _this._changeDetection && _this._changeDetection.markForCheck(); }, 0);
    };
    /**
     * Sync data from view to form model, as well as mark a field as dirty.
     * @param value the field value got from view's input
     * @param editLanguage the current edit language
     */
    BaseFieldEditor.prototype.valueChange = function (value, editLanguage) {
        if (editLanguage === void 0) { editLanguage = '*'; }
        var _a;
        var fieldValue = !this.valueAsDictionary ? value : __assign({}, this.form.value[this.name], (_a = {}, _a[editLanguage] = value, _a));
        _super.prototype.valueChange.call(this, fieldValue);
        this._changeDetection && this._changeDetection.markForCheck();
    };
    BaseFieldEditor.prototype.getValue = function (language) {
        return !this.valueAsDictionary ? this.value : this.value ? this.value[language] : null;
    };
    Object.defineProperty(BaseFieldEditor.prototype, "value", {
        get: function () {
            return this.form.value[this.name];
        },
        set: function (val) {
            this.form.value[this.name] = cloneDeep(val);
            this.onValueSet.emit(cloneDeep(val));
            this._changeDetection && this._changeDetection.markForCheck();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseFieldEditor.prototype, "editLanguage", {
        get: function () {
            return this.isCultureSpecific ? this._editLanguage : '*';
        },
        set: function (language) {
            this._editLanguage = language;
            this._changeDetection && this._changeDetection.markForCheck();
            this.onEditLanguageChanged();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseFieldEditor.prototype, "viewLanguage", {
        get: function () {
            var language = this.editable ? this._viewLanguage : this._previewLanguage;
            return this.isCultureSpecific ? language : '*';
        },
        set: function (language) {
            this._viewLanguage = language;
            this._changeDetection && this._changeDetection.markForCheck();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseFieldEditor.prototype, "previewLanguage", {
        get: function () {
            return this._previewLanguage;
        },
        set: function (language) {
            if (!this._editLanguage) {
                this._editLanguage = language;
            }
            this._previewLanguage = language;
            this._changeDetection && this._changeDetection.markForCheck();
        },
        enumerable: true,
        configurable: true
    });
    BaseFieldEditor.prototype.undo = function () {
        var _a;
        // Note: mark dirty state must be done before updating value in order to
        // make FormFieldContainer to update form's dirty state correctly
        this.onBeforeUndo.emit();
        this.form.controls[this.name].markAsPristine();
        this.form.patchValue((_a = {}, _a[this.name] = cloneDeep(this.backupValue), _a));
        this.onUndo.emit();
        this.onChange.emit(cloneDeep(this.backupValue));
        this._changeDetection && this._changeDetection.markForCheck();
    };
    Object.defineProperty(BaseFieldEditor.prototype, "dirty", {
        get: function () {
            return this.form.controls[this.name].dirty;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseFieldEditor.prototype, "field", {
        get: function () {
            return this;
        },
        enumerable: true,
        configurable: true
    });
    BaseFieldEditor.prototype.onEditLanguageChanged = function () {
        // Note: some children classes can override it after setting edit language (as callback function)
    };
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], BaseFieldEditor.prototype, "valueAsDictionary", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], BaseFieldEditor.prototype, "tooltip", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], BaseFieldEditor.prototype, "onBeforeUndo", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], BaseFieldEditor.prototype, "onUndo", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], BaseFieldEditor.prototype, "onValueSet", void 0);
    __decorate([
        ViewChild('control'),
        __metadata("design:type", ElementRef)
    ], BaseFieldEditor.prototype, "control", void 0);
    __decorate([
        select(['formField', 'previewLanguage']),
        __metadata("design:type", Observable)
    ], BaseFieldEditor.prototype, "previewLanguage$", void 0);
    __decorate([
        select(['formField', 'isEditAllGroup']),
        __metadata("design:type", Observable)
    ], BaseFieldEditor.prototype, "editAll$", void 0);
    return BaseFieldEditor;
}(BaseControl));
export { BaseFieldEditor };
//# sourceMappingURL=base-field-editor.component.js.map