/** Default debounce duration (in ms) */
var DEFAULT_DEBOUNCE_DURATION = 100;
/** Decorates a class method so that it is debounced by the specified duration */
// tslint:disable-next-line:variable-name
export function Debounce(duration) {
    return function (target, key, descriptor) {
        return {
            configurable: true,
            enumerable: descriptor.enumerable,
            get: function getter() {
                // Attach this function to the instance (not the class)
                Object.defineProperty(this, key, {
                    configurable: true,
                    enumerable: descriptor.enumerable,
                    value: _debounce(descriptor.value, duration)
                });
                return this[key];
            }
        };
    };
}
/** Debounces the specified function and returns a wrapper function */
var _debounce = function (method, duration) {
    if (duration === void 0) { duration = DEFAULT_DEBOUNCE_DURATION; }
    var timeoutId;
    function debounceWrapper() {
        var _this = this;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        debounceWrapper['clear']();
        timeoutId = setTimeout(function () {
            timeoutId = null;
            method.apply(_this, args);
        }, duration);
    }
    debounceWrapper['clear'] = function () {
        if (timeoutId) {
            clearTimeout(timeoutId);
            timeoutId = null;
        }
    };
    return debounceWrapper;
};
var ɵ0 = _debounce;
export { ɵ0 };
//# sourceMappingURL=debounce.decorator.js.map