var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Directive, Input, EventEmitter, HostListener, ElementRef } from '@angular/core';
import { map, flatMap, takeUntil } from 'rxjs/operators';
var Draggable = /** @class */ (function () {
    function Draggable(element) {
        this.element = element;
        this._isDraggable = false;
        this._styleInitialized = true;
        this.mouseup = new EventEmitter();
        this.mousedown = new EventEmitter();
        this.mousemove = new EventEmitter();
    }
    Draggable.prototype.onMouseup = function (event) {
        this.mouseup.emit(event);
        this._isDraggable = false;
    };
    Draggable.prototype.onMousedown = function (event) {
        if (this.dragHandle.contains(event.target)) {
            if (this._styleInitialized) {
                this._styleInitialized = false;
                this._setInitalStyle();
            }
            this._isDraggable = true;
            event.preventDefault();
        }
        this.mousedown.emit(event);
    };
    Draggable.prototype.onMousemove = function (event) {
        if (this._isDraggable) {
            this.mousemove.emit(event);
        }
    };
    Draggable.prototype.ngOnInit = function () {
        var _this = this;
        if (!this.dragHandle) {
            this.dragHandle = this.element.nativeElement;
        }
        this.dragHandle.style.cursor = 'move';
        this.mousedrag = this.mousedown.pipe(map(function (event) {
            return {
                top: event.clientY - _this.element.nativeElement.getBoundingClientRect().top,
                left: event.clientX - _this.element.nativeElement.getBoundingClientRect().left,
            };
        }), flatMap(function (imageOffset) { return _this.mousemove.pipe(map(function (pos) { return ({
            top: pos.clientY - imageOffset.top,
            left: pos.clientX - imageOffset.left
        }); }), takeUntil(_this.mouseup)); }));
        this.mousedrag.subscribe({
            next: function (pos) {
                _this.element.nativeElement.style.top = pos.top + 'px';
                _this.element.nativeElement.style.left = pos.left + 'px';
            }
        });
    };
    Draggable.prototype._setInitalStyle = function () {
        var defaultBottomMargin = 10;
        var _a = this.element.nativeElement.getBoundingClientRect(), width = _a.width, height = _a.height, top = _a.top;
        var maxHeight = document.documentElement.clientHeight - top - defaultBottomMargin;
        this.element.nativeElement.style.width = width + 'px';
        this.element.nativeElement.style.height = (height < maxHeight ? height : maxHeight) + 'px';
        this.element.nativeElement.style.position = 'fixed';
        this.element.nativeElement.style.cursor = 'pointer';
    };
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], Draggable.prototype, "dragHandle", void 0);
    __decorate([
        HostListener('document:mouseup', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], Draggable.prototype, "onMouseup", null);
    __decorate([
        HostListener('mousedown', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], Draggable.prototype, "onMousedown", null);
    __decorate([
        HostListener('document:mousemove', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], Draggable.prototype, "onMousemove", null);
    Draggable = __decorate([
        Directive({
            selector: '[draggable]'
        }),
        __metadata("design:paramtypes", [ElementRef])
    ], Draggable);
    return Draggable;
}());
export { Draggable };
//# sourceMappingURL=draggable.directive.js.map