var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Directive, ElementRef, Input } from '@angular/core';
import { Observable, fromEvent } from 'rxjs';
import { debounceTime } from 'rxjs/operators';
import { Debounce } from '../../decorators';
var ImageLazyLoadDirective = /** @class */ (function () {
    function ImageLazyLoadDirective(_element) {
        this._element = _element;
        this._sourceAttributeName = 'image-source';
    }
    ImageLazyLoadDirective.prototype.ngOnInit = function () {
        var _this = this;
        this._onResizeSubscription = fromEvent(window, 'resize').pipe(debounceTime(50)).subscribe(function () { return _this._onScroll(); });
        this._element.nativeElement.addEventListener('scroll', function () { return _this._onScroll(); });
        if (this.loading) {
            this._loadingSubscription = this.loading.subscribe(function (val) { return _this._onScroll(); });
        }
    };
    ImageLazyLoadDirective.prototype.ngOnDestroy = function () {
        this._loadingSubscription && this._loadingSubscription.unsubscribe();
        this._onResizeSubscription && this._onResizeSubscription.unsubscribe();
    };
    ImageLazyLoadDirective.prototype.markForCheck = function () {
        this._onScroll();
    };
    ImageLazyLoadDirective.prototype._onScroll = function () {
        var descendants = this._element.nativeElement.querySelectorAll("img[" + this._sourceAttributeName + "]");
        for (var i = 0; i <= descendants.length - 1; i++) {
            var elem = descendants[i];
            this._onElementScroll(elem, this._sourceAttributeName);
        }
    };
    ImageLazyLoadDirective.prototype._onElementScroll = function (elem, imageAttribute) {
        var rect = elem.getBoundingClientRect();
        var topOfElement = rect.top + document.body.scrollTop;
        var bottomOfElement = topOfElement + elem.offsetHeight;
        var isInViewport = this._isInViewport(topOfElement, bottomOfElement);
        if (isInViewport) {
            elem.setAttribute('src', elem.getAttribute(imageAttribute));
            elem.removeAttribute(imageAttribute);
        }
    };
    ImageLazyLoadDirective.prototype._isInViewport = function (elemTop, elemBotom) {
        var supportPageOffset = window.pageXOffset !== undefined;
        var isCSS1Compat = ((document.compatMode || "") === "CSS1Compat");
        var scrollTop = supportPageOffset ? window.pageYOffset : isCSS1Compat ? document.documentElement.scrollTop : document.body.scrollTop;
        var bottomOfScrollZone = scrollTop + document.body.clientHeight;
        var topOfScrollZone = scrollTop;
        return (bottomOfScrollZone > elemTop) && (topOfScrollZone < elemBotom);
    };
    __decorate([
        Input('imageLazyLoad'),
        __metadata("design:type", Observable)
    ], ImageLazyLoadDirective.prototype, "loading", void 0);
    __decorate([
        Debounce(50),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], ImageLazyLoadDirective.prototype, "_onScroll", null);
    ImageLazyLoadDirective = __decorate([
        Directive({
            selector: '[imageLazyLoad]'
        }),
        __metadata("design:paramtypes", [ElementRef])
    ], ImageLazyLoadDirective);
    return ImageLazyLoadDirective;
}());
export { ImageLazyLoadDirective };
//# sourceMappingURL=image-lazy-load.directive.js.map