var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Directive, ElementRef, Input } from '@angular/core';
import { combineLatest } from 'rxjs';
var LoadingDirective = /** @class */ (function () {
    function LoadingDirective(_element) {
        this._element = _element;
        this._show = false;
    }
    LoadingDirective.prototype.ngOnInit = function () {
        if (this.loading) {
            this._addLoadingIndicator();
        }
    };
    LoadingDirective.prototype.ngOnDestroy = function () {
        this._loadingSubscription && this._loadingSubscription.unsubscribe();
    };
    LoadingDirective.prototype._addLoadingIndicator = function () {
        var _this = this;
        var container = document.createElement('div');
        container.className = 'base__status';
        var loadingElement = document.createElement('div');
        loadingElement.className = 'base__loader';
        loadingElement.insertAdjacentHTML('beforeend', "\n            <i class=\"fa fa-3x fa-circle-o-notch fa-spin\"></i>\n        ");
        loadingElement.style.display = 'none';
        container.appendChild(loadingElement);
        this._element.nativeElement.appendChild(container);
        // this.loading accepts list of Observable<boolean>. For example, FormFieldContainer
        // shows loading indicator both when loading or submitting form. We have this in template:
        // [loading]="[loadingFormSubmit$, loadingFormLoad$]"
        // Using combineLatest and valueSelector, we show loading indicator whether 
        // loadingFormSubmit$ or loadingFormLoad$ has value as true.
        var valueSelector = function () {
            var sources = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                sources[_i] = arguments[_i];
            }
            return sources.some(function (source) { return source; });
        };
        this._loadingSubscription = combineLatest(this.loading, valueSelector).subscribe(function (show) {
            // when having a lot of loading request, to avoid of show and hide loading indicator too many time,
            // we show the indicator immediately, but wait a little bit before hidding it.
            _this._show = show;
            setTimeout(function () { return _this._showLoadingIndicator(loadingElement, _this._show); }, _this._show ? 0 : 100);
        });
    };
    LoadingDirective.prototype._showLoadingIndicator = function (loadingElement, show) {
        loadingElement.style.display = show ? 'block' : 'none';
        if (!show) {
            return;
        }
        var _a = this._element.nativeElement.getBoundingClientRect(), left = _a.left, top = _a.top, width = _a.width, height = _a.height;
        loadingElement.style.left = left + "px";
        loadingElement.style.top = top + "px";
        loadingElement.style.width = width + "px";
        loadingElement.style.height = height + "px";
    };
    __decorate([
        Input('loading'),
        __metadata("design:type", Object)
    ], LoadingDirective.prototype, "loading", void 0);
    LoadingDirective = __decorate([
        Directive({
            selector: '[loading]'
        }),
        __metadata("design:paramtypes", [ElementRef])
    ], LoadingDirective);
    return LoadingDirective;
}());
export { LoadingDirective };
//# sourceMappingURL=loading.directive.js.map