import { AbstractControl, AsyncValidatorFn, FormGroup, ValidatorFn } from '@angular/forms';
import { Observable } from 'rxjs';
import { BaseControl } from '../components';
/**
 * An extension of FormGroup, which introduces the inputMap, in order
 * to be able to get the BaseControl by the property name.
 */
export declare class ExtendedFormGroup extends FormGroup {
    controlMap: {
        [key: string]: AbstractControl;
    };
    inputMap: {
        [key: string]: BaseControl;
    };
    private _skeletonFields;
    private _resetSubject;
    private _afterResetSubject;
    onReset: Observable<any>;
    onAfterReset: Observable<any>;
    private _onPristineSubject;
    onPristine: Observable<any>;
    private _onSubmitSubject;
    onSubmit: Observable<any>;
    private _onSubmitSuccessfulSubject;
    onSubmitSuccessful: Observable<any>;
    private _onSubmitErrorSubject;
    onSubmitError: Observable<any>;
    private _onPostbackSubject;
    onPostback: Observable<any>;
    constructor(controls: {
        [key: string]: AbstractControl;
    }, validator?: ValidatorFn, asyncValidator?: AsyncValidatorFn);
    /**
     * Add a control to this form group.
     * By default, skeleton is false. If it is true, it should be ignored when comparing
     * form's value with backup value.
     */
    addControl(name: string, control: AbstractControl, skeleton?: boolean): void;
    readonly skeletonFields: string[];
    reset(value?: any, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    fireOnSubmit(): void;
    fireOnSubmitSuccessful(): void;
    fireOnSubmitError(): void;
    fireOnPostback(): void;
    setValue(value: {
        [key: string]: any;
    }, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    setControlValue(controlName: string, value: any, compareFunction?: (newValue: any, oldValue: any) => boolean): void;
    markAsPristine(): void;
    /**
     * Removes non-existed field, otherwise an error will be thrown when setting value
     * @param formValue The form value
     */
    removeOrphanControls(formValue: any): void;
    /**
     * Checks the form value and ensures all needed controls are added to the form.
     * @param formValue The form value
     */
    private _ensureControls;
}
