var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { FormControl, FormGroup } from '@angular/forms';
import { Subject } from 'rxjs';
import isEqual from 'lodash-es/isEqual';
import cloneDeep from 'lodash-es/cloneDeep';
/**
 * An extension of FormGroup, which introduces the inputMap, in order
 * to be able to get the BaseControl by the property name.
 */
var ExtendedFormGroup = /** @class */ (function (_super) {
    __extends(ExtendedFormGroup, _super);
    function ExtendedFormGroup(controls, validator, asyncValidator) {
        var _this = _super.call(this, controls, validator, asyncValidator) || this;
        _this.inputMap = {};
        _this._skeletonFields = [];
        _this._resetSubject = new Subject();
        _this._afterResetSubject = new Subject();
        _this.onReset = _this._resetSubject.asObservable();
        _this.onAfterReset = _this._afterResetSubject.asObservable();
        _this._onPristineSubject = new Subject();
        _this.onPristine = _this._onPristineSubject.asObservable();
        _this._onSubmitSubject = new Subject();
        _this.onSubmit = _this._onSubmitSubject.asObservable();
        _this._onSubmitSuccessfulSubject = new Subject();
        _this.onSubmitSuccessful = _this._onSubmitSuccessfulSubject.asObservable();
        _this._onSubmitErrorSubject = new Subject();
        _this.onSubmitError = _this._onSubmitErrorSubject.asObservable();
        _this._onPostbackSubject = new Subject();
        _this.onPostback = _this._onPostbackSubject.asObservable();
        _this.controlMap = controls;
        return _this;
    }
    /**
     * Add a control to this form group.
     * By default, skeleton is false. If it is true, it should be ignored when comparing
     * form's value with backup value.
     */
    ExtendedFormGroup.prototype.addControl = function (name, control, skeleton) {
        if (skeleton === void 0) { skeleton = false; }
        if (skeleton) {
            this._skeletonFields.push(name);
        }
        control['backupValue'] = cloneDeep(control.value);
        _super.prototype.addControl.call(this, name, control);
    };
    Object.defineProperty(ExtendedFormGroup.prototype, "skeletonFields", {
        get: function () {
            return this._skeletonFields;
        },
        enumerable: true,
        configurable: true
    });
    ExtendedFormGroup.prototype.reset = function (value, options) {
        this.markAsPristine(); // to reset cancel and submit buttons
        this._resetSubject.next();
        this._ensureControls(value);
        _super.prototype.reset.call(this, value, options);
        this._afterResetSubject.next();
    };
    ExtendedFormGroup.prototype.fireOnSubmit = function () {
        this._onSubmitSubject.next();
    };
    ExtendedFormGroup.prototype.fireOnSubmitSuccessful = function () {
        this._onSubmitSuccessfulSubject.next();
    };
    ExtendedFormGroup.prototype.fireOnSubmitError = function () {
        this._onSubmitErrorSubject.next();
    };
    ExtendedFormGroup.prototype.fireOnPostback = function () {
        this._onPostbackSubject.next();
    };
    ExtendedFormGroup.prototype.setValue = function (value, options) {
        this._ensureControls(cloneDeep(value));
        _super.prototype.setValue.call(this, cloneDeep(value), options);
    };
    ExtendedFormGroup.prototype.setControlValue = function (controlName, value, compareFunction) {
        if (compareFunction === void 0) { compareFunction = function (newValue, oldValue) { return isEqual(newValue, oldValue); }; }
        var control = this.controls[controlName], backupvalue = control['backupValue'], dirty = !compareFunction(value, backupvalue);
        if (dirty) {
            control.markAsDirty();
        }
        else {
            control.markAsPristine();
        }
        // Note: mark dirty state must be done before updating value in order to
        // make FormFieldContainer to update form's dirty state correctly
        control.setValue(cloneDeep(dirty ? value : backupvalue));
    };
    ExtendedFormGroup.prototype.markAsPristine = function () {
        _super.prototype.markAsPristine.call(this);
        this._onPristineSubject.next();
    };
    /**
     * Removes non-existed field, otherwise an error will be thrown when setting value
     * @param formValue The form value
     */
    ExtendedFormGroup.prototype.removeOrphanControls = function (formValue) {
        var _this = this;
        Object.keys(this.controls).filter(function (c) { return formValue[c] === undefined; }).forEach(function (c) { return _this.removeControl(c); });
    };
    /**
     * Checks the form value and ensures all needed controls are added to the form.
     * @param formValue The form value
     */
    ExtendedFormGroup.prototype._ensureControls = function (formValue) {
        var _this = this;
        if (!formValue) {
            return;
        }
        Object.keys(formValue).filter(function (c) { return !_this.controls[c]; }).forEach(function (c) { return _this.addControl(c, new FormControl(formValue[c])); });
    };
    return ExtendedFormGroup;
}(FormGroup));
export { ExtendedFormGroup };
//# sourceMappingURL=extended-form-group.js.map