import { NgRedux } from '@angular-redux/store';
import { IAppState } from '../stores';
/**
 * Represents the action to manipulate a list, for example:
 * Query items from the list with pagination data.
 * Add, remove items.
 * Update a set of items.
 */
export declare class ListActions {
    private _ngRedux;
    static LIST_SET: string;
    static LIST_QUERY: string;
    static LIST_REMOVE: string;
    static LIST_ADD: string;
    static LIST_UPDATE: string;
    static LIST_MODIFY: string;
    constructor(_ngRedux: NgRedux<IAppState>);
    set(key: string, items?: any[]): {
        type: string;
        payload: {
            key: string;
            items: any[];
        };
    };
    setDispatch(key: string, items?: any[]): void;
    query(key: string, startIndex: number, itemsPerPage: number, filter?: any): {
        type: string;
        payload: {
            key: string;
            startIndex: number;
            itemsPerPage: number;
            filter: any;
        };
    };
    queryDispatch(key: string, startIndex: number, itemsPerPage: number, filter?: any): void;
    remove(key: string, ids: string[], idKey: string): {
        type: string;
        payload: {
            key: string;
            ids: string[];
            idKey: string;
        };
    };
    removeDispatch(key: string, ids: string[], idKey: string): void;
    add(key: string, items: any[]): {
        type: string;
        payload: {
            key: string;
            items: any[];
        };
    };
    addDispatch(key: string, items: any[]): void;
    modify(key: string, lazyLoadedItems: any[]): {
        type: string;
        payload: {
            key: string;
            items: any[];
        };
    };
    /**
     * Updates a set of items.
     * Remark: the input items must be a list of lazy-loaded item, count from zero-based index.
     * To update a specific item, use Update function instead.
     * @param key the key to identify which list should be modified
     * @param lazyLoadedItems a lazy-loaded list to update, from zero-based index
     */
    modifyDispatch(key: string, lazyLoadedItems: any[]): void;
    update(key: string, item: any, idKey: string): {
        type: string;
        payload: {
            key: string;
            item: any;
            idKey: string;
        };
    };
    /**
     * Update a specific item
     * @param key the key to identify which list should be modified
     * @param item item to update
     * @param idKey the Id property name, in order to find the item to update
     */
    updateDispatch(key: string, item: any, idKey: string): void;
}
