var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Injectable } from '@angular/core';
import { NgRedux } from '@angular-redux/store';
/**
 * Represents the action to manipulate a list, for example:
 * Query items from the list with pagination data.
 * Add, remove items.
 * Update a set of items.
 */
var ListActions = /** @class */ (function () {
    function ListActions(_ngRedux) {
        this._ngRedux = _ngRedux;
    }
    ListActions_1 = ListActions;
    ListActions.prototype.set = function (key, items) {
        if (items === void 0) { items = []; }
        return {
            type: ListActions_1.LIST_SET,
            payload: {
                key: key,
                items: items,
            }
        };
    };
    ListActions.prototype.setDispatch = function (key, items) {
        if (items === void 0) { items = []; }
        this._ngRedux.dispatch(this.set(key, items));
    };
    ListActions.prototype.query = function (key, startIndex, itemsPerPage, filter) {
        if (filter === void 0) { filter = null; }
        return {
            type: ListActions_1.LIST_QUERY,
            payload: {
                key: key,
                startIndex: startIndex,
                itemsPerPage: itemsPerPage,
                filter: filter,
            }
        };
    };
    ListActions.prototype.queryDispatch = function (key, startIndex, itemsPerPage, filter) {
        if (filter === void 0) { filter = null; }
        this._ngRedux.dispatch(this.query(key, startIndex, itemsPerPage, filter));
    };
    ListActions.prototype.remove = function (key, ids, idKey) {
        return {
            type: ListActions_1.LIST_REMOVE,
            payload: {
                key: key,
                ids: ids,
                idKey: idKey,
            }
        };
    };
    ListActions.prototype.removeDispatch = function (key, ids, idKey) {
        this._ngRedux.dispatch(this.remove(key, ids, idKey));
    };
    ListActions.prototype.add = function (key, items) {
        return {
            type: ListActions_1.LIST_ADD,
            payload: {
                key: key,
                items: items,
            }
        };
    };
    ListActions.prototype.addDispatch = function (key, items) {
        this._ngRedux.dispatch(this.add(key, items));
    };
    ListActions.prototype.modify = function (key, lazyLoadedItems) {
        return {
            type: ListActions_1.LIST_MODIFY,
            payload: {
                key: key,
                items: lazyLoadedItems,
            }
        };
    };
    /**
     * Updates a set of items.
     * Remark: the input items must be a list of lazy-loaded item, count from zero-based index.
     * To update a specific item, use Update function instead.
     * @param key the key to identify which list should be modified
     * @param lazyLoadedItems a lazy-loaded list to update, from zero-based index
     */
    ListActions.prototype.modifyDispatch = function (key, lazyLoadedItems) {
        this._ngRedux.dispatch(this.modify(key, lazyLoadedItems));
    };
    ListActions.prototype.update = function (key, item, idKey) {
        return {
            type: ListActions_1.LIST_UPDATE,
            payload: {
                key: key,
                item: item,
                idKey: idKey
            }
        };
    };
    /**
     * Update a specific item
     * @param key the key to identify which list should be modified
     * @param item item to update
     * @param idKey the Id property name, in order to find the item to update
     */
    ListActions.prototype.updateDispatch = function (key, item, idKey) {
        this._ngRedux.dispatch(this.update(key, item, idKey));
    };
    var ListActions_1;
    ListActions.LIST_SET = 'LIST_SET';
    ListActions.LIST_QUERY = 'LIST_QUERY';
    ListActions.LIST_REMOVE = 'LIST_REMOVE';
    ListActions.LIST_ADD = 'LIST_ADD';
    ListActions.LIST_UPDATE = 'LIST_UPDATE';
    ListActions.LIST_MODIFY = 'LIST_MODIFY';
    ListActions = ListActions_1 = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [NgRedux])
    ], ListActions);
    return ListActions;
}());
export { ListActions };
//# sourceMappingURL=list.actions.js.map