/**
 * A reducer can be reused across application. For instance, the TreeView is a generic view.
 * We have the Folder tree, CMS page tree and Product category tree. All of them are tree views.
 * And they all use the treeReducer, but different instance, each maintain different portion of state.
 * To accomplish this, we need to have the namedReducer, which map a reducer to a name.
 * When an action is invoked, the input 'name' will be checked to make sure the reducer is updating
 * its portion of state.
 * More info: http://redux.js.org/docs/recipes/reducers/ReusingReducerLogic.html
 */
export function namedReducer(reducer, name) {
    return function (state, action) {
        var reducerName = action.reducerName;
        var isInitializationCall = state === undefined;
        if (reducerName !== NAMED_ANY && name.indexOf(reducerName) === -1 && !isInitializationCall) {
            return state;
        }
        return reducer(state, action);
    };
}
/**
 * The 'any' reducer name, which is used to forward action to all named reducers,
 * regarless what reducer name is.
 */
export var NAMED_ANY = 'any';
export var NAMED_BASE_ENTITY_LIST_KEYS_FIELD_LIST = 'NAMED_BASE_ENTITY_LIST_KEYS_FIELD_LIST';
//# sourceMappingURL=named.reducer.js.map